// src/util/withHighlight.ts
import {
  H as NodeH,
  HIGHLIGHT_REQUEST_HEADER
} from "@highlight-run/node";
var H = {
  init: (options) => {
    if (!NodeH.isInitialized()) {
      NodeH.init(options);
    }
  },
  isInitialized: () => NodeH.isInitialized(),
  metrics: (metrics, opts) => {
    const h = global.__HIGHLIGHT__;
    if (h && !opts) {
      opts = h;
    }
    if (!(opts == null ? void 0 : opts.secureSessionId)) {
      return console.warn(
        "H.metrics session could not be inferred the handler context."
      );
    }
    for (const m of metrics) {
      NodeH.recordMetric(
        opts.secureSessionId,
        m.name,
        m.value,
        opts.requestId,
        m.tags
      );
    }
  }
};
var Highlight = (options) => (origHandler) => {
  return async (req, res) => {
    if (!NodeH.isInitialized()) {
      NodeH.init(options);
    }
    const processHighlightHeaders = () => {
      if (req.headers && req.headers[HIGHLIGHT_REQUEST_HEADER]) {
        const [secureSessionId2, requestId2] = `${req.headers[HIGHLIGHT_REQUEST_HEADER]}`.split("/");
        if (secureSessionId2 && requestId2) {
          ;
          global.__HIGHLIGHT__ = { secureSessionId: secureSessionId2, requestId: requestId2 };
          return { secureSessionId: secureSessionId2, requestId: requestId2 };
        }
      }
      return { secureSessionId: void 0, requestId: void 0 };
    };
    const { secureSessionId, requestId } = processHighlightHeaders();
    const start = new Date();
    try {
      return await origHandler(req, res);
    } catch (e) {
      NodeH.consumeEvent(secureSessionId);
      if (e instanceof Error) {
        NodeH.consumeError(e, secureSessionId, requestId);
        await NodeH.flush();
      }
      res.statusCode = 500;
      res.statusMessage = "Internal Server Error";
      throw e;
    } finally {
      const delta = (new Date().getTime() - start.getTime()) * 1e6;
      const { secureSessionId: secureSessionId2, requestId: requestId2 } = processHighlightHeaders();
      if (secureSessionId2) {
        NodeH.recordMetric(
          secureSessionId2,
          "latency",
          delta,
          requestId2
        );
        await NodeH.flush();
      }
    }
  };
};

// src/util/highlightWebpackPlugin.ts
import { uploadSourcemaps } from "@highlight-run/sourcemap-uploader/dist/lib";
var HighlightWebpackPlugin = class {
  constructor(apiKey, appVersion, path, basePath) {
    this.apiKey = apiKey;
    this.appVersion = appVersion;
    this.path = path;
    this.basePath = basePath;
  }
  apply(compiler) {
    compiler.hooks.afterEmit.tap("HighlightWebpackPlugin", () => {
      uploadSourcemaps({
        apiKey: this.apiKey,
        appVersion: this.appVersion,
        path: this.path,
        basePath: this.basePath
      });
    });
    return compiler;
  }
};

// src/util/withHighlightConfig.ts
var getDefaultOpts = async (config, highlightOpts) => {
  const isProdBuild = process.env.NODE_ENV === "production";
  const hasSourcemapApiKey = !!process.env.HIGHLIGHT_SOURCEMAP_UPLOAD_API_KEY || !!(highlightOpts == null ? void 0 : highlightOpts.apiKey);
  let version = null;
  if (config.generateBuildId) {
    const buildId = config.generateBuildId();
    if (typeof buildId === "string") {
      version = buildId;
    } else {
      version = await buildId;
    }
  }
  return {
    uploadSourceMaps: isProdBuild && ((highlightOpts == null ? void 0 : highlightOpts.uploadSourceMaps) ?? (!config.productionBrowserSourceMaps && hasSourcemapApiKey)),
    configureHighlightProxy: (highlightOpts == null ? void 0 : highlightOpts.configureHighlightProxy) ?? true,
    apiKey: (highlightOpts == null ? void 0 : highlightOpts.apiKey) ?? "",
    appVersion: (highlightOpts == null ? void 0 : highlightOpts.appVersion) ?? version ?? "",
    sourceMapsPath: (highlightOpts == null ? void 0 : highlightOpts.sourceMapsPath) ?? ".next/",
    sourceMapsBasePath: (highlightOpts == null ? void 0 : highlightOpts.sourceMapsBasePath) ?? "_next/"
  };
};
var getHighlightConfig = async (config, highlightOpts) => {
  const defaultOpts = await getDefaultOpts(config, highlightOpts);
  let newRewrites = config.rewrites;
  if (defaultOpts.uploadSourceMaps || defaultOpts.configureHighlightProxy) {
    newRewrites = async () => {
      let re;
      if (!config.rewrites) {
        re = new Array();
      } else {
        re = await config.rewrites();
      }
      const sourcemapRewrite = {
        source: "/:path*.map",
        destination: "/404"
      };
      const highlightRewrite = {
        source: "/highlight-events",
        destination: "https://pub.highlight.run"
      };
      if (!re || Array.isArray(re)) {
        return {
          beforeFiles: defaultOpts.uploadSourceMaps ? [sourcemapRewrite] : [],
          afterFiles: defaultOpts.configureHighlightProxy ? (re ?? []).concat(highlightRewrite) : [],
          fallback: []
        };
      } else {
        return {
          beforeFiles: defaultOpts.uploadSourceMaps ? (re.beforeFiles ?? []).concat(sourcemapRewrite) : re.beforeFiles,
          afterFiles: defaultOpts.configureHighlightProxy ? (re.afterFiles ?? []).concat(highlightRewrite) : re.afterFiles,
          fallback: re.fallback
        };
      }
    };
  }
  let newWebpack = config.webpack;
  if (defaultOpts.uploadSourceMaps) {
    newWebpack = (webpackConfig, opts) => {
      let originalConfig = webpackConfig;
      if (config.webpack) {
        originalConfig = config.webpack(webpackConfig, opts);
      }
      originalConfig.plugins.push(
        new HighlightWebpackPlugin(
          defaultOpts.apiKey,
          defaultOpts.appVersion,
          defaultOpts.sourceMapsPath,
          defaultOpts.sourceMapsBasePath
        )
      );
      return originalConfig;
    };
  }
  return {
    ...config,
    productionBrowserSourceMaps: defaultOpts.uploadSourceMaps || config.productionBrowserSourceMaps,
    rewrites: newRewrites,
    webpack: newWebpack
  };
};
var withHighlightConfig = async (config, highlightOpts) => {
  if (typeof config === "function") {
    return async (phase, { defaultConfig }) => {
      const userNextConfigObject = config(phase, { defaultConfig });
      if (typeof userNextConfigObject === "function") {
        const nc = await userNextConfigObject;
        return await getHighlightConfig(nc, highlightOpts);
      } else {
        return await getHighlightConfig(
          userNextConfigObject,
          highlightOpts
        );
      }
    };
  } else {
    return await getHighlightConfig(config, highlightOpts);
  }
};
export {
  H,
  Highlight,
  withHighlightConfig
};
